/**
 * Title:        BCJR Decoder Tester
 * Copyright:    Copyright (c) 2001, Ling Li
 * Company:      California Institute of Technology
 * @author Ling Li
 * @email  ling@cs.caltech.edu
 * @version 1.1
 */

import java.io.*;
import ecc.*;

public class TestBCJR {
    final static int trunc_len = 4;
    public static void main(String[] args) {
        int N = 1024;
        int x[], i, len;

        len = N;
        x = new int[len];
        x[0] = 1; x[1] = x[2] = x[3] = x[4] = x[5] = 0;


        for (i = 6; i < len; i++)
            x[i] = x[i-5] ^ x[i-6];
        Random rnd = new Random();
        rnd.setSeed(529);
        for (i = 0; i < len; i++)
            x[i] = (rnd.urand1() > 0.5)? 1 : 0;

        TurboCode code = new BCJR_sqr();
        int y[][] = code.encode(x, trunc_len);

        Channel ch = new Channel(111);
        double sr[][];
        int sx[];

        for (int stni = 12; stni > 1; stni--) {
            double stn = stni * 0.5;
            double sigma = Math.exp(- stn * Math.log(10) / 20);
            //double sigma2 = Math.exp(- (stn-1) * Math.log(10) / 20);

            int ssum = 0;
            String filename = "llr_" + String.valueOf(stn) + ".dat";
            try {
                FileWriter f = new FileWriter(filename);
                for (int trial = 0; trial < 5000; trial++) {
                    sr = ch.AWGN(y, sigma);
                    sx = code.decode(sr, trunc_len, sigma, 0.5);
                    double LLR[] = code.getLLR();

                    for (i = 0; i < sx.length; i++) {
                        if (x[i] != sx[i])
                            ssum++;
                        f.write(String.valueOf(LLR[i] * (1-2*x[i])));
                        f.write('\n');
                    }
                    sx = null;
                }
                f.close();
            } catch (IOException e) {
                System.out.println("Error saveing " + filename);
            }
            System.out.print("avg # of errs = ");
            System.out.print(ssum / 5000.0);
            System.out.print(", Eb/N0 = ");
            System.out.println(stn);
//            System.out.print("dB, sigma = ");
//            System.out.println(sigma);
        }
    }
}