/**
 * Title:        Random Number Generator
 * Copyright:    Copyright (c) 2001, Ling Li
 * Company:      California Institute of Technology
 * @author Ling Li
 * @email  ling@cs.caltech.edu
 * @version 1.0
 */

package ecc;


public class Random {
    final int IA = 16807;
    final int IM = 2147483647;
    final double AM = 1.0 / IM;
    final int IQ = 127773;
    final int IR = 2836;
    final int NTAB = 32;
    final int NDIV = 1 + (IM-1)/NTAB;

    private int idum;
    private int iy;
    private int iv[];

    public Random() {
        iv = new int[NTAB];
        setSeed(218);
    }

    public void setSeed(int seed) {
        idum = seed;

        if (idum < 1) idum = 1;

        for (int j = NTAB + 7; j>=0; j--) {
            int k = idum / IQ;
            idum = IA * (idum - k*IQ) - IR * k;
            if (idum < 0) idum += IM;

            if (j < NTAB) iv[j] = idum;
        }

        iy=iv[0];
    }

    public int getSeed() {
        return idum;
    }

    public double urand0() {
        int k = idum / IQ;
        idum = IA * (idum - k*IQ) - k*IR;
        if (idum < 0) idum += IM;

        return (AM * idum);
    }

    public double urand1() {
        int k = idum / IQ;
        idum = IA * (idum - k*IQ) - IR * k;
        if (idum < 0) idum += IM;

        int j = iy / NDIV;
        iy = iv[j];
        iv[j] = idum;

        return AM * iy;
    }
}